--[[
************************************************************************

questlog.lua

File date: 2019-11-19T22:30:05Z
File revision: 
Project revision: 274
Project version: r274


Author: Angelike

************************************************************************
]]--

local beql = LibStub("AceAddon-3.0"):GetAddon("beql")
local L = LibStub("AceLocale-3.0"):GetLocale("beql")

function beql:initQuestlog()
    beql:saveLayout(QuestLogFrame);
    beql:saveLayout(QuestLogFrameCloseButton);
    beql:saveLayout(QuestFrameExitButton);
    beql:saveLayout(QuestFramePushQuestButton);
    beql:saveLayout(QuestLogListScrollFrame);
    beql:saveLayout(QuestLogDetailScrollFrame);
    beql:saveLayout(QuestLogCountRight);

    QuestLogListScrollFrame.buttons = {};
    QuestLogFrame.textures = {};
    QuestLogFrame.QUESTS_DISPLAYED = QUESTS_DISPLAYED;

    for _, child in ipairs({QuestLogFrame:GetChildren()}) do
        if(child:GetName() and child:GetName():match('QuestLogTitle')) then
            beql:initQuestLogTitleButton(child);
        end
    end

    for _, region in ipairs({QuestLogFrame:GetRegions()}) do
        if (region:IsObjectType("Texture")) then
            local name = region:GetTexture():match('(Icon)$') or region:GetTexture():match('([^\-]*t)$');

            if (name) then
                QuestLogFrame.textures[lcfirst(name)] = region;
            end
        end
    end

    QuestLogFrame.UIPanelWindows = {
        area = UIPanelWindows.QuestLogFrame.area,
        width = UIPanelWindows.QuestLogFrame.width,
        height = UIPanelWindows.QuestLogFrame.height
    };
end

function beql:initBeqlQuestlog()
    beql:saveLayout(beql.frames.questLogAnchor);

    for i = 1, 25 do
        local frame = _G['QuestLogTitle' .. i];
        if (frame == nil) then
            frame = CreateFrame('Button', 'QuestLogTitle' .. i, QuestLogFrame, 'QuestLogTitleButtonTemplate');
            beql:initQuestLogTitleButton(frame);
            frame:ClearAllPoints();
            frame:SetPoint('TOPLEFT', _G['QuestLogTitle' .. (i - 1)], 'BOTTOMLEFT');
            frame:SetID(i);
            frame:Hide();
        end
    end

    local t = {
        {'Interface\\AddOns\\beql\\Images\\LeftOn.tga', 0},
        {'Interface\\AddOns\\beql\\Images\\RightOn.tga', 512}
    };

    for _, tInfo in ipairs(t) do
        local texture = QuestLogFrame:CreateTexture(nil, 'BORDER');
        texture:SetTexture(tInfo[1]);
        texture:SetParent(QuestLogFrame);
        texture:SetHeight(512)
        texture:SetWidth(512)
        texture:ClearAllPoints();
        texture:SetPoint('TOPLEFT', QuestLogFrame, 'TOPLEFT', tInfo[2], 0);
        texture:Hide();

        QuestLogFrame.textures['beql' .. tInfo[1]:match('(%w*)On.tga$')] = texture;
    end

    -- QuestLogDetailScrollFrame:Show is triggered by QuestLogUpdate, so we must place a hoot to keep it hidden
    hooksecurefunc(QuestLogDetailScrollFrame, 'Show', function(self) 
        if(beql:IsEnabled() and beql.db.profile.QuestLog.style == 'extended' and beql.db.char.QuestLogMinimized) then 
            self:Hide();
        end 
    end);

    hooksecurefunc('ToggleQuestLog', beql.questLog_OnToggle);
    hooksecurefunc(_G.QuestLogDummyText, 'SetText', beql.QuestLogDummyTextSetText);

    beql.frames.questWatchFrame:SetScript('OnDragStart', beql.questLogFrame_OnDragStart);
    beql.frames.questWatchFrame:SetScript('OnDragStop', beql.questLogFrame_OnDragStop);
end

function beql:restoreBlizzardQuestlog()
    for i = QuestLogFrame.QUESTS_DISPLAYED + 1, QUESTS_DISPLAYED do
        local frame = _G['QuestLogTitle' .. i];
        if (frame ~= nil) then
            frame:Hide();
        end
    end

    QUESTS_DISPLAYED = QuestLogFrame.QUESTS_DISPLAYED;

	-- frame Defaults
	QuestLogTitleText:SetText(QUEST_LOG)
	beql:QuestLog_SetFontSize(12)
	QuestLogFrame:SetAlpha(1)
	QuestLogFrame:SetScale(1)
	QuestLogFrame:RegisterForDrag(0)

    beql:restoreLayout(QuestLogFrame);
    beql:restoreLayout(QuestLogFrameCloseButton);
    beql:restoreLayout(QuestFrameExitButton);
    beql:restoreLayout(QuestFramePushQuestButton);
    beql:restoreLayout(QuestLogListScrollFrame);
    beql:restoreLayout(QuestLogDetailScrollFrame);
    beql:restoreLayout(QuestLogCountRight);

	-- Hide Addon Buttons
	beql.frames.questLogToggleButton:Hide();
	beql.frames.configButton:Hide();

    QuestLogDetailScrollFrame:Show();

    QuestLogFrame.textures.icon:SetTexture("Interface\\QuestFrame\\UI-QuestLog-BookIcon");
	-- restore old textures
	for k, texture in pairs(QuestLogFrame.textures) do
		if (k:match('beql')) then
			texture:Hide()
		else
			texture:Show()
		end
	end

	UIPanelWindows.QuestLogFrame.area = QuestLogFrame.UIPanelWindows.area;
	UIPanelWindows.QuestLogFrame.area = QuestLogFrame.UIPanelWindows.area;
	UIPanelWindows.QuestLogFrame.area = QuestLogFrame.UIPanelWindows.area;

	-- Blizzard Status is now resored, activate beql stuff if needed
end

function beql:QuestLog_ExtendedQuestLog()
    QUESTS_DISPLAYED = 25;

    QuestLogFrame:SetAlpha(beql.db.profile.QuestLog.Alpha);
	QuestLogFrame:SetScale(beql.db.profile.QuestLog.Scale);

    -- adjust button's position
    QuestLogFrameCloseButton:ClearAllPoints();
    QuestLogFrameCloseButton:SetPoint('TOPRIGHT', QuestLogFrame, 'TOPRIGHT', 2, -8);

    QuestFrameExitButton:ClearAllPoints();
    QuestFrameExitButton:SetPoint('BOTTOMRIGHT', QuestLogFrame, 'BOTTOMRIGHT', -10, 55);

    QuestFramePushQuestButton:ClearAllPoints();
    QuestFramePushQuestButton:SetPoint('TOPLEFT', QuestLogFrameAbandonButton, 'TOPRIGHT', 2);

    QuestLogCountRight:ClearAllPoints();
    QuestLogCountRight:SetPoint('TOPRIGHT', QuestLogFrame, 'TOPRIGHT', -15, -41);

    QuestLogFrame:SetAttribute("UIPanelLayout-height", 562)
    QuestLogFrame:SetHeight(562)

	-- Code from DoubleWide by Iriel
	-- {
	QuestLogFrame:SetAttribute("UIPanelLayout-width", 680);
	QuestLogFrame:SetAttribute("UIPanelLayout-height", 562);
	QuestLogFrame:SetAttribute("UIPanelLayout-area", 'doublewide');

	QuestLogListScrollFrame:SetHeight(410)

	for key, texture in pairs(QuestLogFrame.textures) do
        if (key:match('beql') or key:match('icon')) then
            texture:Show()
        else
            texture:Hide()
        end  
	end

    QuestLogFrame.textures.icon:SetTexture("Interface\\Addons\\beql\\Images\\icon");

	-- }
	-- Code from DoubleWide by Iriel, mod by angelike @ wowace.com

end

function beql:questLog_OnToggle()
    local style = beql.db.profile.QuestLog.style;

    if (style == 'simple' or beql:IsEnabled() ~= true) then
        beql:restoreBlizzardQuestlog();
    end

    if(not beql:IsEnabled() or QuestLogFrame:IsVisible() == false)then
        return;
    end

    QuestLogTitleText:SetText(L["Bayi's Extended Quest Log"])
    beql.frames.configButton:Show();

    QuestLogFrame:ClearAllPoints();
    QuestLogFrame:SetPoint("TOPLEFT", beql.frames.questLogAnchor, "TOPLEFT", 0,0)

    -- movability
	if (beql.db.profile.QuestLog.locked ~= true) then
        QuestLogFrame:EnableMouse(true);
		QuestLogFrame:RegisterForDrag("LeftButton");
	end

    if (beql.db.profile.QuestLog.style ~= 'simple') then
        beql.frames.questLogToggleButton:Show();
        beql:QuestLog_ExtendedQuestLog();

        if(GetNumQuestLogEntries() == 0 or style == 'minimized' or style == 'extended' and beql.db.char.QuestLogMinimized) then
            beql:QuestLog_Minimize();
        elseif (style == 'maximized' or style == 'extended' and not beql.db.char.QuestLogMinimized) then
            beql:QuestLog_Maximize();
        end
    end

    beql:QuestLog_SetFontSize(beql.db.profile.QuestLog.FontSize);

	QuestLog_Update();
end

function beql:QuestLog_Minimize(button)
	beql.db.char.QuestLogMinimized = true
	button = button or beql.frames.questLogToggleButton
	button:SetNormalTexture("Interface\\AddOns\\beql\\Images\\restore_up")
	button:SetPushedTexture("Interface\\AddOns\\beql\\Images\\restore_down")

	QuestLogFrame:SetAttribute("UIPanelLayout-width", 384)
	QuestLogFrame:SetWidth(353)

	-- change textures
	QuestLogFrame.textures.beqlRight:Hide();
	QuestLogFrame.textures.beqlLeft:SetTexture("Interface\\AddOns\\beql\\Images\\LeftOff");

    QuestLogDetailScrollFrame:ClearAllPoints();
	QuestLogDetailScrollFrame:SetPoint("TOPLEFT", QuestLogListScrollFrame, "TOPRIGHT", 30, 2);
	QuestLogDetailScrollFrame:SetHeight(410);
	QuestLogDetailScrollFrame:Hide();
end

function beql:QuestLog_Maximize(button)
	beql.db.char.QuestLogMinimized = false

	if ( GetNumQuestLogEntries() == 0 ) then
		-- no quests in log, do not allow to maximize
		return
	end

	button = button and button or beql.frames.questLogToggleButton
	button:SetNormalTexture("Interface\\AddOns\\beql\\Images\\minimize_up")
	button:SetPushedTexture("Interface\\AddOns\\beql\\Images\\minimize_down")
	QuestLogFrame:SetAttribute("UIPanelLayout-width", 680)
	QuestLogFrame:SetWidth(682)


	-- change textures
	QuestLogFrame.textures.beqlRight:Show();
	QuestLogFrame.textures.beqlLeft:SetTexture("Interface\\AddOns\\beql\\Images\\LeftOn");

    QuestLogDetailScrollFrame:ClearAllPoints();
	QuestLogDetailScrollFrame:SetPoint("TOPLEFT", QuestLogListScrollFrame, "TOPRIGHT", 30, 2);
	QuestLogDetailScrollFrame:SetHeight(410);
    QuestLogDetailScrollFrame:Show();
end

function beql:initQuestLogTitleButton(button)
    tinsert(QuestLogListScrollFrame.buttons, button);

    for _, region in ipairs( {button:GetRegions()}) do
        if(region:GetName()) then
            local name = region:GetName():gsub(button:GetName(), '');
            local key = name:gsub('^.', strlower(name:match('^.'))); -- lower first char
            button[key] = region;
        end
    end

    -- save handler
    button._OnClick = button:GetScript('OnClick');

    button:SetScript('OnClick', beql.TileButtonOnClick)
    hooksecurefunc(button, 'SetText', beql.TileButtonSetText);
end


function beql:QuestLog_SetFontSize()
    local fonts = {};

    for key, v in pairs(_G) do
        if ((key:match('^QuestInfo') or key:match('^QuestLog')) and type(v) == 'table' and v.GetFont and not key:match('Watch')) then
            tinsert(fonts, v);
        end
    end

    for _, font in ipairs(fonts) do 
        if (font._heightFactor == nil) then
            font._heightFactor = select(2, font:GetFont()) / 12;
        end

        local fontName, fontHeight, fontFlags = font:GetFont();
        fontHeight = beql.db.profile.QuestLog.FontSize * font._heightFactor;
        font:SetFont(fontName, fontHeight, fontFlags);
    end
end


function beql:questLogFrame_OnDragStart()
    if (beql.db.profile.QuestLog.locked) then
        return;
    end

    if (beql.frames.questLogAnchor:IsMovable()) then
        beql.frames.questLogAnchor.ismoving = true;
        beql.frames.questLogAnchor:StartMoving();
    end
end

function beql:questLogFrame_OnDragStop()
    beql.frames.questLogAnchor.ismoving = false
    beql.frames.questLogAnchor:StopMovingOrSizing()
    beql:SaveFramePosition(beql.frames.questLogAnchor)
end

--
-- Hooks
--

function beql:TileButtonOnClick(...)
    local button = self;

    if (not IsShiftKeyDown() or not beql:IsEnabled() or beql.db.profile.disabledtracker) then
        securecall(button._OnClick, button, ...);

        if (beql.db.profile.QuestLog.style == 'extended') then 
            beql:QuestLog_Maximize();
        end 

        return;
    end

    beql:toggleQuestWatch(button:GetID() + FauxScrollFrame_GetOffset(QuestLogListScrollFrame));
end

function beql:TileButtonSetText()
    local button = self;

    if(button.hookCall or not beql:IsEnabled()) then
        return;
    end

    button.hookCall = true;

    local questIndex = button:GetID() + FauxScrollFrame_GetOffset(QuestLogListScrollFrame);
    local spacing = button:GetText():match('^[ ]*');
    local text = GetQuestLogTitle(questIndex);
    local tag = beql:GetQuestTag(questIndex);
    local newText = spacing .. tag .. text;

    button:SetText(newText);
    QuestLogDummyText.taggedText = newText;

    button.hookCall = nil;
end

function beql:QuestLogDummyTextSetText(text)
    local button = self;

    if (button.hookCall or button.taggedText == nil) then
        return;
    end

    button.hookCall = true;
    button:SetText(button.taggedText);
    button.taggedText = nil;
    button.hookCall = nil;
end


function beql:GetQuestTag(index)
	local _, level, questTag, isHeader, _, _, frequency = GetQuestLogTitle(index);

    local tags = {};

	if (beql.db.profile.QuestLog.showlevel and level > 0) then
        tinsert(tags, level);
    end

	-- build a tagged title
	if (beql.db.profile.QuestLog.showTags) then
        if (questTag == GROUP) then
            tinsert(tags, 'G');
        elseif (questTag == ELITE) then
            tinsert(tags, '+');
        elseif (questTag == ITEM_HEROIC) then
            tinsert(tags, 'H');
        elseif (questTag == RAID) then
            tinsert(tags, 'R');
        elseif (questTag == PVP) then
            tinsert(tags, 'P');
        elseif (frequency == 2) then
            tinsert(tags, 'Y');
        elseif (frequency == 3) then
            tinsert(tags, 'W');
        elseif (questTag == LFG_TYPE_DUNGEON) then
            tinsert(tags, 'D');
        end
    end

    if (#tags > 0) then
        tinsert(tags, 1, '[');
        tinsert(tags, '] ');
    end

	return table.concat(tags, '');
end

function beql:restoreLayout(region)
    if (not region or not region.GetNumPoints) then
        return;
    end 

    region:ClearAllPoints();

    for _, point in ipairs(region._points) do
        region:SetPoint(unpack(point));
    end

    region:SetWidth(region._width);
    region:SetHeight(region._height);
end

function beql:saveLayout(region)
    if (not region or not region.GetNumPoints) then
        return;
    end 

    region._points = {};

    for i = 1, region:GetNumPoints() do
        tinsert(region._points, {region:GetPoint(i)});
    end

    region._width = region:GetWidth();
    region._height = region:GetHeight();
end


--- EOF --- 
 
